import { Vis1DInputData } from '../subPlots/types';
import { getCommonLayout, getCSSVariableHSL, sharedTickFont } from '../prepareLayout/LayoutCommon';
import { PlotType } from 'plotly.js';
import { scaleOrdinal, scaleQuantize } from 'd3';
import { visualizationColors } from 'ts-common';

export const preparePlotDataAndLayoutHistogramCount = (
  xData: Vis1DInputData,
  yData: Vis1DInputData,
  zData: Vis1DInputData,
  showAxis: boolean,
  stackVariable: boolean,
) => {
  const primaryColor = getCSSVariableHSL('--clr-sec--400');
  const mainColors = visualizationColors.GPCat.colors[14];

  const isCategorical = typeof xData.data[0] === 'string';

  const frequencyMap = xData.data.reduce(
    (acc, item, index) => {
      if (!acc[item]) acc[item] = { count: 0, zValueCounts: {} };
      acc[item].count += (yData.data?.[index] as number) ?? 1;
      if (zData.data) {
        const zValue = zData.data[index] || '(Empty)';
        acc[item].zValueCounts[zValue] = (acc[item].zValueCounts[zValue] || 0) + 1;
      }
      return acc;
    },
    {} as Record<string, { count: number; zValueCounts: Record<string, number> }>,
  );

  const sortedCategories = Object.entries(frequencyMap).sort((a, b) => b[1].count - a[1].count);
  const sortedLabels = sortedCategories.map(([label]) => label);

  const traces = (() => {
    if (isCategorical) {
      return [
        {
          type: 'bar' as PlotType,
          x: sortedLabels,
          y: sortedCategories.map(([_, data]) => data.count),
          marker: { color: primaryColor },
        },
      ];
    } else {
      return [
        {
          type: 'histogram' as PlotType,
          x: xData.data,
          marker: { color: primaryColor },
        },
      ];
    }
  })();

  const layout: Partial<Plotly.Layout> = getCommonLayout({
    showAxis,
    xAxisLabel: xData.label,
    yAxisLabel: yData.label,
    xValues: xData.data,
    yValues: yData.data,
    plotType: 'bar',
    xAxisData: xData.data,
    sortedLabels,
    sharedTickFont,
  });

  return { data: traces, layout };
};
