import React, { useEffect, useRef, useState } from 'react';
import Plot from 'react-plotly.js';
import { Vis1DInputData } from './types';
import { preparePlotDataAndLayoutPie } from '../prepareData/preparePlotPie';

type Vis1DPieProps = {
  xData: Vis1DInputData;
  showAxis: boolean;
};

export const Vis1DPie: React.FC<Vis1DPieProps> = ({ xData, showAxis }) => {
  // common to all vis
  const internalRef = useRef<HTMLDivElement>(null);
  const [divSize, setDivSize] = useState({ width: 0, height: 0 });

  useEffect(() => {
    const handleResize = () => {
      if (internalRef.current) {
        const { width, height } = internalRef.current.getBoundingClientRect();
        setDivSize({ width, height });
      }
    };

    handleResize();
    window.addEventListener('resize', handleResize);
    if (internalRef.current) {
      new ResizeObserver(handleResize).observe(internalRef.current);
    }

    return () => {
      window.removeEventListener('resize', handleResize);
    };
  }, []);

  // specific to this vis
  const { data, layout } = preparePlotDataAndLayoutPie(xData, showAxis);

  return (
    <div className="h-full w-full flex items-center justify-center overflow-hidden relative" ref={internalRef}>
      <Plot
        data={data}
        config={{
          responsive: true,
          scrollZoom: false,
          displayModeBar: false,
          displaylogo: false,
        }}
        layout={{
          ...layout,
          width: divSize.width,
          height: divSize.height,
          dragmode: false,
        }}
      />
    </div>
  );
};
